.project-detail-container[data-v-fe390362]{padding:32px;background-color:#f0f2f5;position:relative;margin-bottom:10px}.project-detail-container .detail-wrapper[data-v-fe390362]{background:#fff;padding:16px 16px 0;margin-bottom:32px}@media (max-width:1024px){.detail-wrapper[data-v-fe390362]{padding:8px}}.el-row[data-v-fe390362]{margin-bottom:10px}.el-row[data-v-fe390362]:last-child{margin-bottom:0}.content-key[data-v-fe390362]{font-weight:700;font-size:smaller}