.createPost-container[data-v-a4bfa87e]{position:relative}.createPost-container .createPost-main-container[data-v-a4bfa87e]{padding:40px 45px 20px 50px}.createPost-container .createPost-main-container .postInfo-form[data-v-a4bfa87e]{margin:0 auto;text-align:center;width:400px}.createPost-container .createPost-main-container .postInfo-form-item[data-v-a4bfa87e]{position:relative;margin-bottom:10px;float:left}.createPost-container .createPost-main-container .postInfo-form-item[data-v-a4bfa87e]:after{content:"";display:table;clear:both}