.panel-group[data-v-7677d7e3]{margin-top:18px}.panel-group .card-panel-col[data-v-7677d7e3]{margin-bottom:32px}.panel-group .card-panel[data-v-7677d7e3]{height:108px;cursor:pointer;font-size:12px;position:relative;overflow:hidden;color:#666;background:#fff;-webkit-box-shadow:4px 4px 40px rgba(0,0,0,.05);box-shadow:4px 4px 40px rgba(0,0,0,.05);border-color:rgba(0,0,0,.05)}.panel-group .card-panel:hover .card-panel-icon-wrapper[data-v-7677d7e3]{color:#fff}.panel-group .card-panel:hover .icon-people[data-v-7677d7e3]{background:#40c9c6}.panel-group .card-panel:hover .icon-message[data-v-7677d7e3]{background:#36a3f7}.panel-group .card-panel:hover .icon-money[data-v-7677d7e3]{background:#f4516c}.panel-group .card-panel:hover .icon-shopping[data-v-7677d7e3]{background:#34bfa3}.panel-group .card-panel .icon-people[data-v-7677d7e3]{color:#40c9c6}.panel-group .card-panel .icon-message[data-v-7677d7e3]{color:#36a3f7}.panel-group .card-panel .icon-money[data-v-7677d7e3]{color:#f4516c}.panel-group .card-panel .icon-shopping[data-v-7677d7e3]{color:#34bfa3}.panel-group .card-panel .card-panel-icon-wrapper[data-v-7677d7e3]{float:left;margin:14px 0 0 14px;padding:16px;-webkit-transition:all .38s ease-out;transition:all .38s ease-out;border-radius:6px}.panel-group .card-panel .card-panel-icon[data-v-7677d7e3]{float:left;font-size:48px}.panel-group .card-panel .card-panel-description[data-v-7677d7e3]{float:right;font-weight:700;margin:26px;margin-left:0}.panel-group .card-panel .card-panel-description .card-panel-text[data-v-7677d7e3]{line-height:18px;color:rgba(0,0,0,.45);font-size:16px;margin-bottom:12px}.panel-group .card-panel .card-panel-description .card-panel-num[data-v-7677d7e3]{font-size:20px}@media (max-width:550px){.card-panel-description[data-v-7677d7e3]{display:none}.card-panel-icon-wrapper[data-v-7677d7e3]{float:none!important;width:100%;height:100%;margin:0!important}.card-panel-icon-wrapper .svg-icon[data-v-7677d7e3]{display:block;margin:14px auto!important;float:none!important}}.dashboard-editor-container[data-v-733199c5]{padding:32px;background-color:#f0f2f5;position:relative}.dashboard-editor-container .github-corner[data-v-733199c5]{position:absolute;top:0;border:0;right:0}.dashboard-editor-container .chart-wrapper[data-v-733199c5]{background:#fff;padding:16px 16px 0;margin-bottom:32px}@media (max-width:1024px){.chart-wrapper[data-v-733199c5]{padding:8px}}.emptyGif[data-v-f2b782e4]{display:block;width:45%;margin:0 auto}.dashboard-editor-container[data-v-f2b782e4]{background-color:#e3e3e3;min-height:100vh;padding:50px 60px 0}.dashboard-editor-container .pan-info-roles[data-v-f2b782e4]{font-size:12px;font-weight:700;color:#333;display:block}.dashboard-editor-container .info-container[data-v-f2b782e4]{position:relative;margin-left:190px;height:150px;line-height:200px}.dashboard-editor-container .info-container .display_name[data-v-f2b782e4]{font-size:48px;line-height:48px;color:#212121;position:absolute;top:25px}