.icons-container[data-v-b82194d6]{margin:10px 20px 0;overflow:hidden}.icons-container .icon-item[data-v-b82194d6]{margin:20px;height:100px;text-align:center;width:125px;float:left;font-size:50px;color:#24292e;cursor:pointer}.icons-container span[data-v-b82194d6]{display:block;font-size:16px;margin-top:10px}.icons-container .disabled[data-v-b82194d6]{pointer-events:none}