.yaml-editor[data-v-1e6e3673]{height:100%;position:relative}.yaml-editor[data-v-1e6e3673] .CodeMirror{font-size:16px;line-height:150%;height:auto}.yaml-editor[data-v-1e6e3673] .CodeMirror-scroll{min-height:300px;height:auto}.createPost-container[data-v-7c929b30]{position:relative}.createPost-container .createPost-main-container[data-v-7c929b30]{padding:10px 45px 20px 50px}.createPost-container .createPost-main-container .postInfo-container[data-v-7c929b30]{position:relative;margin-bottom:5px}.createPost-container .createPost-main-container .postInfo-container[data-v-7c929b30]:after{content:"";display:table;clear:both}.createPost-container .createPost-main-container .postInfo-container .postInfo-container-item[data-v-7c929b30]{float:left}.createPost-container .word-counter[data-v-7c929b30]{width:40px;position:absolute;right:10px;top:0}.article-textarea[data-v-7c929b30] textarea{padding-right:40px;resize:none;border:none;border-radius:0;border-bottom:1px solid #bfcbd9}